@@ -1,15 +1,6 @@
 #' @title Create basemapData object for basemap plotting
 #' @description Internal function to create a \code{basemapData} object for \code{\link{basemap}}
-#' @param limits See \code{\link{basemap}}
-#' @param data See \code{\link{basemap}}
-#' @param shapefiles See \code{\link{basemap}}
-#' @param bathymetry See \code{\link{basemap}}
-#' @param glaciers See \code{\link{basemap}}
-#' @param resolution See \code{\link{basemap}}
-#' @param lon.interval,lat.interval See \code{\link{basemap}}
-#' @param expand.factor See \code{\link{auto_limits}}
-#' @param rotate See \code{\link{basemap}}
-#' @param verbose See \code{\link{basemap}}
+#' @inheritParams basemap
 #' @details This is an internal function, which is automatically run by the \code{\link{basemap}} function. Common users do not need to worry about these details.
 #' @return A list of class \code{basemapData} containing information required for plotting a \code{\link{basemap}}.
 #' @keywords internal
@@ -19,9 +10,8 @@
 #' @seealso \code{\link{basemap}}
 
 # Test paramters
-# limits = c(-180, 180, -60, 60)
-# limits = NULL; data = NULL; shapefiles = NULL; bathymetry = FALSE; glaciers = FALSE; resolution = "low"; lon.interval = NULL; lat.interval = NULL; expand.factor = 1.1; rotate = FALSE; verbose = TRUE
-basemap_data <- function(limits = NULL, data = NULL, shapefiles = NULL, bathymetry = FALSE, glaciers = FALSE, resolution = "low", lon.interval = NULL, lat.interval = NULL, expand.factor = 1.1, rotate = FALSE, verbose = TRUE) {
+# limits = NULL; data = NULL; shapefiles = NULL; bathymetry = FALSE; glaciers = FALSE; lon.interval = NULL; lat.interval = NULL; expand.factor = 1.1; rotate = FALSE; verbose = TRUE
+basemap_data <- function(limits = NULL, data = NULL, shapefiles = NULL, bathymetry = FALSE, glaciers = FALSE, lon.interval = NULL, lat.interval = NULL, expand.factor = 1.1, rotate = FALSE, verbose = FALSE) {
   
   # Switches and checks ####
   
@@ -40,7 +30,7 @@ basemap_data <- function(limits = NULL, data = NULL, shapefiles = NULL, bathymet
       ## Load the shapefiles if download required
       
       if(!is.na(shapefiles$path)) {
-        tmp <- load_map_data(x = shapefiles) 
+        tmp <- load_map_data(x = shapefiles)
         
         shapefiles <- lapply(shapefiles, function(k) {
           test <- which(names(tmp) == k)
@@ -48,7 +38,7 @@ basemap_data <- function(limits = NULL, data = NULL, shapefiles = NULL, bathymet
           if(length(test) != 1) {
             k
           } else {
-            tmp[[test]] 
+            tmp[[test]]
           }
         })
         
@@ -66,7 +56,7 @@ basemap_data <- function(limits = NULL, data = NULL, shapefiles = NULL, bathymet
     }
   }
   
-  # 2. limits argument dictates the limits and defines shapefile if not specified
+  # 2. limits argument dictates the limits and defines shapefile if not specified ###
   
   if(!is.null(limits)) {
     
@@ -89,7 +79,7 @@ basemap_data <- function(limits = NULL, data = NULL, shapefiles = NULL, bathymet
       }
       
     } else if(
-      (all(abs(limits[1:2]) == 180) & diff(limits[3:4]) <= 60) | 
+      (all(abs(limits[1:2]) == 180) & diff(limits[3:4]) <= 60) |
       all(abs(limits[1:2]) == 0)
     ) { ## Fix the problem with limits ending up on a single line in polar maps
       
@@ -137,18 +127,20 @@ basemap_data <- function(limits = NULL, data = NULL, shapefiles = NULL, bathymet
           stop("Cannot detect the required shapefiles automatically from projected limits coordinates. Change the limits to decimal degrees, provide data with decimal degree information or specify the shapefiles argument.")
         }
         
-        clipLimits <- 
-          auto_limits(data = 
+        clipLimits <-
+          auto_limits(data =
                         expand.grid(data.frame(
-                          lon = limits[1:2], 
+                          lon = limits[1:2],
                           lat = limits[3:4])
-                        ), 
-                      lon = "lon", lat = "lat", 
-                      proj.in = 
-                        ifelse(class(shapefiles$land) == "SpatialPolygonsDataFrame",
-                               suppressWarnings(sp::proj4string(shapefiles$land)),
-                               suppressWarnings(sp::proj4string(eval(parse(text = shapefiles$land))))),
-                      proj.out = "EPSG:4326",
+                        ),
+                      lon = "lon", lat = "lat",
+                      proj.in =
+                        if(class(shapefiles$land) == "SpatialPolygonsDataFrame") {
+                          raster::crs(shapefiles$land)
+                        } else {
+                          convert_crs(shapefiles$crs)
+                        },
+                      proj.out = 4326,
                       verbose = verbose)
         
       } else { # Limits given as decimal degrees
@@ -167,25 +159,25 @@ basemap_data <- function(limits = NULL, data = NULL, shapefiles = NULL, bathymet
         midLon <- deg_to_dd(midLon)
         
         if(is.null(shapefiles)) {
-          clipLimits <- 
+          clipLimits <-
             auto_limits(data = expand.grid(
-              lon = sort(c(limits[1:2], midLon)), 
+              lon = sort(c(limits[1:2], midLon)),
               lat = limits[3:4]),
               lon = "lon", lat = "lat",
-              verbose = !rotate)
+              verbose = verbose)
         } else {
-          clipLimits <- 
+          clipLimits <-
             auto_limits(data = expand.grid(
-              lon = sort(c(limits[1:2], midLon)), 
+              lon = sort(c(limits[1:2], midLon)),
               lat = limits[3:4]),
               lon = "lon", lat = "lat",
-              proj.out = 
-                ifelse(grepl("SpatialPolygons", class(shapefiles$land)), 
-                       sp::proj4string(shapefiles$land), 
+              proj.out =
+                ifelse(grepl("SpatialPolygons", class(shapefiles$land)),
+                       sp::proj4string(shapefiles$land),
                        sp::proj4string(eval(parse(text = shapefiles$land)))
                 ),
-              verbose = !rotate)
-        } 
+              verbose = verbose)
+        }
         
       }
     }
@@ -197,15 +189,15 @@ basemap_data <- function(limits = NULL, data = NULL, shapefiles = NULL, bathymet
       shapefile.name <- shapefile.def$shapefile.name
     }
     
-  } 
+  }
   
-  # 3. data argument defines the limits and shapefiles if not specified. Also helps the limits argument to find shapefile if it was given as projected coordinates ####
+  # 3. data argument defines the limits and shapefiles if not specified. Also helps the limits argument to find shapefile if it was given as projected coordinates ###
   
   if(!is.null(data) & is.null(limits)) {
     
     if(rotate) {
       
-      clipLimits <- auto_limits(data, verbose = !rotate)
+      clipLimits <- auto_limits(data, verbose = verbose)
       limits <- clipLimits$ddLimits
       decLimits <- is_decimal_limit(limits)
       
@@ -223,17 +215,17 @@ basemap_data <- function(limits = NULL, data = NULL, shapefiles = NULL, bathymet
     } else {
       
       if(!is.null(shapefiles)) {
-        clipLimits <- 
-          auto_limits(data, verbose = !rotate, 
-                      proj.out = 
-                        suppressWarnings(sp::proj4string(shapefiles$land))
-                      ) 
+        clipLimits <-
+          auto_limits(data, verbose = verbose,
+                      proj.out = raster::crs(shapefiles$land),
+                      expand.factor = expand.factor
+          )
       } else {
-        clipLimits <- auto_limits(data, expand.factor = 1.1, verbose = !rotate)
+        clipLimits <- auto_limits(data, expand.factor = expand.factor, verbose = verbose)
       }
       
       limits <- clipLimits$ddLimits
-      decLimits <- is_decimal_limit(limits)  
+      decLimits <- is_decimal_limit(limits)
       
     }
     
@@ -247,7 +239,7 @@ basemap_data <- function(limits = NULL, data = NULL, shapefiles = NULL, bathymet
     }
   }
   
-  # 4. Define the shapefiles to be used in the map ###
+  # 4. Define the shapefiles to be used in the map ####
   
   if(!shapefilesDefined) {
     
@@ -276,7 +268,7 @@ basemap_data <- function(limits = NULL, data = NULL, shapefiles = NULL, bathymet
   
   ##  Define the CRS for the underlying data ###
   
-  LandCRS <- suppressWarnings(sp::proj4string(shapefiles$land))
+  LandCRS <- sf::st_crs(shapefiles$land) # old: raster::crs(shapefiles$land)
   
   # 5. Crop and rotate shapefiles if needed ###
   
@@ -296,26 +288,24 @@ basemap_data <- function(limits = NULL, data = NULL, shapefiles = NULL, bathymet
     
     if(all(!tmp)) stop("No shapefile layers to plot.")
     
+    
     tmp <- lapply(shapefiles[names(tmp[tmp])], function(k) {
-      out <- raster::extent(k)
-      matrix(out[1:4], nrow = 1)
+      # out <- raster::extent(k)
+      matrix(sf::st_bbox(k), nrow = 1)
     })
     
     tmp <- as.data.frame(do.call(rbind, tmp))
-    map.limits <- c(min(tmp[1]), max(tmp[2]), min(tmp[3]), max(tmp[4]))
+    map.limits <- c(min(tmp[1]), max(tmp[3]), min(tmp[2]), max(tmp[4]))
     
-    clipLimits <- 
+    clipLimits <-
       auto_limits(data = expand.grid(
-        lon = c(tmp[[1]], tmp[[2]]), 
-        lat = c(tmp[[3]], tmp[[4]])), 
-        lon = "lon", lat = "lat",
-        proj.in = LandCRS, proj.out = "+init=epsg:4326")
-    
-    # Clip the shapefiles (Clipping likely not required. Untick if it is.)
-    
-    # shapefiles$land <- clip_shapefile(shapefiles$land, limits = limits, proj4.limits = ifelse(decLimits, "+init=epsg:4326", sp::proj4string(shapefiles$land)))
-    # if(glaciers) shapefiles$glacier <- clip_shapefile(shapefiles$glacier, limits = limits, proj4.limits = ifelse(decLimits, "+init=epsg:4326", sp::proj4string(shapefiles$land)))
-    # if(bathymetry) shapefiles$bathy <- clip_shapefile(shapefiles$bathy, limits = limits, proj4.limits = ifelse(decLimits, "+init=epsg:4326", sp::proj4string(shapefiles$land)))
+        lon = c(tmp[[1]], tmp[[3]]),
+        lat = c(tmp[[2]], tmp[[4]])),
+        lon = "lon",
+        lat = "lat",
+        proj.in = LandCRS,
+        proj.out = 4326
+      )
     
   } else if(polarMap) { # Polar maps
     
@@ -323,11 +313,14 @@ basemap_data <- function(limits = NULL, data = NULL, shapefiles = NULL, bathymet
     
     # Clip the shapefiles
     
-    landBoundary <- clip_shapefile(shapefiles$land, limits = limits, proj.limits = "+init=epsg:4326", return.boundary = TRUE)
+    landBoundary <- clip_shapefile(shapefiles$land, 
+                                   limits = limits, 
+                                   proj.limits = convert_crs(4326), 
+                                   return.boundary = TRUE)
     
     shapefiles$land <- landBoundary$shapefile
-    if(glaciers) shapefiles$glacier <- clip_shapefile(shapefiles$glacier, limits = limits, proj.limits = ifelse(decLimits, "+init=epsg:4326", LandCRS))
-    if(bathymetry) shapefiles$bathy <- clip_shapefile(shapefiles$bathy, limits = limits, proj.limits = ifelse(decLimits, "+init=epsg:4326", LandCRS))
+    if(glaciers) shapefiles$glacier <- clip_shapefile(shapefiles$glacier, limits = limits, proj.limits = ifelse(decLimits, convert_crs(4326), LandCRS))
+    if(bathymetry) shapefiles$bathy <- clip_shapefile(shapefiles$bathy, limits = limits, proj.limits = ifelse(decLimits, convert_crs(4326), LandCRS))
     
     # Define map limits
     
@@ -345,25 +338,29 @@ basemap_data <- function(limits = NULL, data = NULL, shapefiles = NULL, bathymet
     ## Rotate the shapefiles
     
     if(rotate) {
-      LandCRS <- gsub("lon_0=0", paste0("lon_0=", midLon), LandCRS)
-      clipLimits <- auto_limits(data = 
+      LandCRS <- sf::st_crs(gsub("lon_0=0", paste0("lon_0=", midLon), LandCRS$proj4string))
+      clipLimits <- auto_limits(data =
                                   expand.grid(
-                                    lon = sort(c(limits[1:2], midLon)), 
+                                    lon = sort(c(limits[1:2], midLon)),
                                     lat = limits[3:4]),
                                 lon = "lon", lat = "lat",
                                 proj.out = LandCRS)
       
-      shapefiles$land <- suppressWarnings(sp::spTransform(shapefiles$land, CRSobj = suppressWarnings(sp::CRS(LandCRS))))
+      # (These conversions will disappear once the package is entirely sf based)
+      shapefiles$land <- 
+        sf::as_Spatial( 
+          sf::st_transform(sf::st_as_sf(shapefiles$land), LandCRS)
+        )
       
-      if(glaciers) shapefiles$glacier <- suppressWarnings(sp::spTransform(shapefiles$glacier, CRSobj = suppressWarnings(sp::CRS(LandCRS))))
-      if(bathymetry) shapefiles$bathy <- suppressWarnings(sp::spTransform(shapefiles$bathy, CRSobj = suppressWarnings(sp::CRS(LandCRS))))
+      if(glaciers) shapefiles$glacier <- sf::as_Spatial(sf::st_transform(sf::st_as_sf(shapefiles$glacier), LandCRS))
+      if(bathymetry) shapefiles$bathy <- sf::as_Spatial(sf::st_transform(sf::st_as_sf(shapefiles$bathy), LandCRS))
       
     }
     
     # Clip the shapefiles
     
-    shapefiles$land <- clip_shapefile(shapefiles$land, 
-                                      limits = clipLimits$projBound, 
+    shapefiles$land <- clip_shapefile(shapefiles$land,
+                                      limits = clipLimits$projBound,
                                       proj.limits = clipLimits$proj.out
     )
     
@@ -385,7 +382,7 @@ basemap_data <- function(limits = NULL, data = NULL, shapefiles = NULL, bathymet
   
   if(exists("clipLimits")) {
     if(abs(clipLimits$ddLimits[4]) != 90) {
-      tmp <- sp::spTransform(clipLimits$projBound, sp::CRS(SRS_string = "EPSG:4326"))@bbox
+      tmp <- sp::spTransform(clipLimits$projBound, sp::CRS(convert_crs(4326)))@bbox
       clipLimits$ddLimits <- unname(c(sort(tmp[1,]), sort(tmp[2,])))
     }
   }
